package com.xisc.pm.app.service.imports;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xisc.pm.domain.entity.PmHtolStatic;
import com.xisc.pm.domain.entity.PmHtolStaticTra;
import com.xisc.pm.domain.repository.PmHtolStaticRepository;
import com.xisc.pm.domain.repository.PmHtolStaticTraRepository;
import io.choerodon.core.exception.CommonException;
import io.choerodon.core.oauth.DetailsHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.hzero.boot.imported.app.service.IBatchImportService;
import org.hzero.boot.imported.infra.validator.annotation.ImportService;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author 纪文桃 wentao.ji01@hand-china.com
 * @since 2024/7/26 10:39
 * 已废弃
 */
@Slf4j
@ImportService(templateCode = "PM.HTOL_STATIC")
public class PmHtolStaticImportImpl implements IBatchImportService {


    private final ObjectMapper objectMapper;

    private final PmHtolStaticRepository pmHtolStaticRepository;

    private final PmHtolStaticTraRepository pmHtolStaticTraRepository;

    public PmHtolStaticImportImpl(ObjectMapper objectMapper, PmHtolStaticRepository pmHtolStaticRepository, PmHtolStaticTraRepository pmHtolStaticTraRepository) {
        this.objectMapper = objectMapper;
        this.pmHtolStaticRepository = pmHtolStaticRepository;
        this.pmHtolStaticTraRepository = pmHtolStaticTraRepository;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean doImport(List<String> data) {
        // 如果集合为空，则返回
        if (CollectionUtils.isEmpty(data)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 获取租户id和当前时间以及用户信息
        Long tenantId = 0L;
        Date now = new Date();
        String username = DetailsHelper.getUserDetails().getUsername();
        if (DetailsHelper.getUserDetails() != null && DetailsHelper.getUserDetails().getTenantId() != null) {
            tenantId = DetailsHelper.getUserDetails().getTenantId();
        }
        // 定义要插入的List
        List<PmHtolStatic> list = new ArrayList<>(data.size());
        List<PmHtolStaticTra> tralist = new ArrayList<>(data.size());

        // 数据转对象
        for (String json : data) {
            try {
                PmHtolStatic pmHtolStatic = objectMapper.readValue(json, PmHtolStatic.class);
                pmHtolStatic.setTenantId(tenantId);
                list.add(pmHtolStatic);

            } catch (JsonProcessingException e) {
                log.error(e.getMessage(), e);
            }
        }

        // 如果集合为空，则结束
        if (CollectionUtils.isEmpty(list)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 插入表
        pmHtolStaticRepository.batchInsertSelective(list);

        // 获取履历表数据
        for (PmHtolStatic pmHtolStatic : list) {
            PmHtolStaticTra pmHtolStaticTra = new PmHtolStaticTra();
            BeanUtils.copyProperties(pmHtolStatic, pmHtolStaticTra);
            // 设置操作类型为新增（I），并记录操作时间和用户
            pmHtolStaticTra.setOperationType("I");
            pmHtolStaticTra.setOperationTime(now);
            pmHtolStaticTra.setOperationUser(username);
            tralist.add(pmHtolStaticTra);
        }
        // 插入履历表
        pmHtolStaticTraRepository.batchInsertSelective(tralist);
        return true;
    }
}
