package com.xisc.pm.app.service.imports;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xisc.pm.domain.entity.PmIngotSpecTrimBt;
import com.xisc.pm.domain.entity.PmIngotSpecTrimBtTra;
import com.xisc.pm.domain.repository.PmIngotSpecTrimBtRepository;
import com.xisc.pm.domain.repository.PmIngotSpecTrimBtTraRepository;
import io.choerodon.core.exception.CommonException;
import io.choerodon.core.oauth.DetailsHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.hzero.boot.imported.app.service.IBatchImportService;
import org.hzero.boot.imported.infra.validator.annotation.ImportService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author 纪文桃 wentao.ji01@hand-china.com
 * @date 2024/12/9 16:56
 * @description
 */
@Slf4j
@ImportService(templateCode = "PM.INGOT_SPEC_TRIM_BT")
public class PmIngotSpecTrimBtImportImpl implements IBatchImportService {

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private PmIngotSpecTrimBtRepository pmIngotSpecTrimBtRepository;

    @Autowired
    private PmIngotSpecTrimBtTraRepository pmIngotSpecTrimBtTraRepository;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean doImport(List<String> data) {
        // 如果集合为空，则返回
        if (CollectionUtils.isEmpty(data)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 获取租户id和当前时间以及用户信息
        Long tenantId = 0L;
        Date now = new Date();
        String username = DetailsHelper.getUserDetails().getUsername();
        if (DetailsHelper.getUserDetails() != null && DetailsHelper.getUserDetails().getTenantId() != null) {
            tenantId = DetailsHelper.getUserDetails().getTenantId();
        }
        // 定义要插入的List
        List<PmIngotSpecTrimBt> list = new ArrayList<>(data.size());
        List<PmIngotSpecTrimBtTra> tralist = new ArrayList<>(data.size());

        // 数据转对象
        for (String json : data) {
            try {
                PmIngotSpecTrimBt pmIngotSpecTrimBt = objectMapper.readValue(json, PmIngotSpecTrimBt.class);
                pmIngotSpecTrimBt.setTenantId(tenantId);
                list.add(pmIngotSpecTrimBt);

            } catch (JsonProcessingException e) {
                log.error(e.getMessage(), e);
            }
        }

        // 如果集合为空，则结束
        if (CollectionUtils.isEmpty(list)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 插入表
        pmIngotSpecTrimBtRepository.batchInsertSelective(list);

        // 获取履历表数据
        for (PmIngotSpecTrimBt pmIngotSpecTrimBt : list) {
            PmIngotSpecTrimBtTra pmIngotSpecTrimBtTra = new PmIngotSpecTrimBtTra();
            BeanUtils.copyProperties(pmIngotSpecTrimBt, pmIngotSpecTrimBtTra);
            // 设置操作类型为新增（I），并记录操作时间和用户
            pmIngotSpecTrimBtTra.setOperationType("I");
            pmIngotSpecTrimBtTra.setOperationTime(now);
            pmIngotSpecTrimBtTra.setOperationUser(username);
            tralist.add(pmIngotSpecTrimBtTra);
        }
        // 插入履历表
        pmIngotSpecTrimBtTraRepository.batchInsertSelective(tralist);
        return true;
    }
}
