package com.xisc.pm.app.service.imports;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xisc.pm.domain.entity.PmSlabReplBt;
import com.xisc.pm.domain.entity.PmSlabReplBtTra;
import com.xisc.pm.domain.repository.PmSlabReplBtRepository;
import com.xisc.pm.domain.repository.PmSlabReplBtTraRepository;
import io.choerodon.core.exception.CommonException;
import io.choerodon.core.oauth.DetailsHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.hzero.boot.imported.app.service.IBatchImportService;
import org.hzero.boot.imported.infra.validator.annotation.ImportService;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author 纪文桃 wentao.ji01@hand-china.com
 * @since 2024/7/26 10:13
 */
@Slf4j
@ImportService(templateCode = "PM.SLAB_REPL_BT")
public class PmSlabReplBtImportImpl implements IBatchImportService {

    private final ObjectMapper objectMapper;

    private final PmSlabReplBtRepository pmSlabReplBtRepository;

    private final PmSlabReplBtTraRepository pmSlabReplBtTraRepository;

    public PmSlabReplBtImportImpl(ObjectMapper objectMapper, PmSlabReplBtRepository pmSlabReplBtRepository, PmSlabReplBtTraRepository pmSlabReplBtTraRepository) {
        this.objectMapper = objectMapper;
        this.pmSlabReplBtRepository = pmSlabReplBtRepository;
        this.pmSlabReplBtTraRepository = pmSlabReplBtTraRepository;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean doImport(List<String> data) {
        // 如果集合为空，则返回
        if (CollectionUtils.isEmpty(data)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 获取租户id和当前时间以及用户信息
        Long tenantId = 0L;
        Date now = new Date();
        String username = DetailsHelper.getUserDetails().getUsername();
        if (DetailsHelper.getUserDetails() != null && DetailsHelper.getUserDetails().getTenantId() != null) {
            tenantId = DetailsHelper.getUserDetails().getTenantId();
        }
        // 定义要插入的List
        List<PmSlabReplBt> list = new ArrayList<>(data.size());
        List<PmSlabReplBtTra> tralist = new ArrayList<>(data.size());

        // 数据转对象
        for (String json : data) {
            try {
                PmSlabReplBt pmSlabReplBt = objectMapper.readValue(json, PmSlabReplBt.class);
                pmSlabReplBt.setTenantId(tenantId);
                list.add(pmSlabReplBt);

            } catch (JsonProcessingException e) {
                log.error(e.getMessage(), e);
            }
        }

        // 如果集合为空，则结束
        if (CollectionUtils.isEmpty(list)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 插入表
        pmSlabReplBtRepository.batchInsertSelective(list);

        // 获取履历表数据
        for (PmSlabReplBt pmSlabReplBt : list) {
            PmSlabReplBtTra pmSlabReplBtTra = new PmSlabReplBtTra();
            BeanUtils.copyProperties(pmSlabReplBt, pmSlabReplBtTra);
            // 设置操作类型为新增（I），并记录操作时间和用户
            pmSlabReplBtTra.setOperationType("I");
            pmSlabReplBtTra.setOperationTime(now);
            pmSlabReplBtTra.setOperationUser(username);
            tralist.add(pmSlabReplBtTra);
        }
        // 插入履历表
        pmSlabReplBtTraRepository.batchInsertSelective(tralist);
        return true;
    }
}
