package com.xisc.pm.app.service.imports;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xisc.pm.domain.entity.PmStCodeBt;
import com.xisc.pm.domain.entity.PmStCodeBtTra;
import com.xisc.pm.domain.repository.PmStCodeBtRepository;
import com.xisc.pm.domain.repository.PmStCodeBtTraRepository;
import io.choerodon.core.exception.CommonException;
import io.choerodon.core.oauth.DetailsHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.hzero.boot.imported.app.service.IBatchImportService;
import org.hzero.boot.imported.infra.validator.annotation.ImportService;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author 纪文桃 wentao.ji01@hand-china.com
 * @since 2024/9/25 15:13
 */
@Slf4j
@ImportService(templateCode = "PM.ST_CODE_BT")
public class PmStCodeBtImportImpl implements IBatchImportService {

    private final ObjectMapper objectMapper;

    private final PmStCodeBtRepository pmStCodeBtRepository;

    private final PmStCodeBtTraRepository pmStCodeBtTraRepository;

    public PmStCodeBtImportImpl(ObjectMapper objectMapper, PmStCodeBtRepository pmStCodeBtRepository, PmStCodeBtTraRepository pmStCodeBtTraRepository) {
        this.objectMapper = objectMapper;
        this.pmStCodeBtRepository = pmStCodeBtRepository;
        this.pmStCodeBtTraRepository = pmStCodeBtTraRepository;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean doImport(List<String> data) {
        // 如果集合为空，则返回
        if (CollectionUtils.isEmpty(data)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 获取租户id和当前时间以及用户信息
        Long tenantId = 0L;
        Date now = new Date();
        String username = DetailsHelper.getUserDetails().getUsername();
        if (DetailsHelper.getUserDetails() != null && DetailsHelper.getUserDetails().getTenantId() != null) {
            tenantId = DetailsHelper.getUserDetails().getTenantId();
        }
        // 定义要插入的List
        List<PmStCodeBt> list = new ArrayList<>(data.size());
        List<PmStCodeBtTra> tralist = new ArrayList<>(data.size());

        // 数据转对象
        for (String json : data) {
            try {
                PmStCodeBt pmStCodeBt = objectMapper.readValue(json, PmStCodeBt.class);
                pmStCodeBt.setTenantId(tenantId);
                list.add(pmStCodeBt);

            } catch (JsonProcessingException e) {
                log.error(e.getMessage(), e);
            }
        }

        // 如果集合为空，则结束
        if (CollectionUtils.isEmpty(list)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 插入表
        pmStCodeBtRepository.batchInsertSelective(list);

        // 获取履历表数据
        for (PmStCodeBt pmStCodeBt : list) {
            PmStCodeBtTra pmStCodeBtTra = new PmStCodeBtTra();
            BeanUtils.copyProperties(pmStCodeBt, pmStCodeBtTra);
            // 设置操作类型为新增（I），并记录操作时间和用户
            pmStCodeBtTra.setOperationType("I");
            pmStCodeBtTra.setOperationTime(now);
            pmStCodeBtTra.setOperationUser(username);
            tralist.add(pmStCodeBtTra);
        }
        // 插入履历表
        pmStCodeBtTraRepository.batchInsertSelective(tralist);
        return true;
    }
}
