package com.xisc.pm.app.service.imports;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xisc.pm.domain.entity.PmThickAddvBt;
import com.xisc.pm.domain.entity.PmThickAddvBtTra;
import com.xisc.pm.domain.repository.PmThickAddvBtRepository;
import com.xisc.pm.domain.repository.PmThickAddvBtTraRepository;
import io.choerodon.core.exception.CommonException;
import io.choerodon.core.oauth.DetailsHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.hzero.boot.imported.app.service.IBatchImportService;
import org.hzero.boot.imported.infra.validator.annotation.ImportService;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author 纪文桃 wentao.ji01@hand-china.com
 * @since 2024/9/23 18:09
 */
@Slf4j
@ImportService(templateCode = "PM.THICK_ADDV_BT")
public class PmThickAddvBtImportImpl implements IBatchImportService {

    private PmThickAddvBtRepository pmThickAddvBtRepository;

    private PmThickAddvBtTraRepository pmThickAddvBtTraRepository;

    private ObjectMapper objectMapper;

    public PmThickAddvBtImportImpl(PmThickAddvBtRepository pmThickAddvBtRepository,
                                   PmThickAddvBtTraRepository pmThickAddvBtTraRepository, ObjectMapper objectMapper) {
        this.pmThickAddvBtRepository = pmThickAddvBtRepository;
        this.pmThickAddvBtTraRepository = pmThickAddvBtTraRepository;
        this.objectMapper = objectMapper;
    }

    @Override
    public Boolean doImport(List<String> data) {
        // 如果集合为空，则返回
        if (CollectionUtils.isEmpty(data)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 获取租户id和当前时间以及用户信息
        Long tenantId = 0L;
        Date now = new Date();
        String username = DetailsHelper.getUserDetails().getUsername();
        if (DetailsHelper.getUserDetails() != null && DetailsHelper.getUserDetails().getTenantId() != null) {
            tenantId = DetailsHelper.getUserDetails().getTenantId();
        }
        // 定义要插入的List
        List<PmThickAddvBt> list = new ArrayList<>(data.size());
        List<PmThickAddvBtTra> tralist = new ArrayList<>(data.size());

        // 数据转对象
        for (String json : data) {
            try {
                PmThickAddvBt pmThickAddvBt = objectMapper.readValue(json, PmThickAddvBt.class);
                pmThickAddvBt.setTenantId(tenantId);
                list.add(pmThickAddvBt);

            } catch (JsonProcessingException e) {
                log.error(e.getMessage(), e);
                ;
            }
        }

        // 如果集合为空，则结束
        if (CollectionUtils.isEmpty(list)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 插入表
        pmThickAddvBtRepository.batchInsertSelective(list);

        // 获取履历表数据
        for (PmThickAddvBt pmThickAddvBt : list) {
            PmThickAddvBtTra pmThickAddvBtTra = new PmThickAddvBtTra();
            BeanUtils.copyProperties(pmThickAddvBt, pmThickAddvBtTra);
            // 设置操作类型为新增（I），并记录操作时间和用户
            pmThickAddvBtTra.setOperationType("I");
            pmThickAddvBtTra.setOperationTime(now);
            pmThickAddvBtTra.setOperationUser(username);
            tralist.add(pmThickAddvBtTra);
        }
        // 插入履历表
        pmThickAddvBtTraRepository.batchInsertSelective(tralist);
        return true;
    }
}
