package com.xisc.pm.app.service.strategy.pdrule;

import com.xisc.pm.api.dto.RuleParamDTO;
import com.xisc.pm.app.service.strategy.RuleExecStrategyFactory;
import com.xisc.pm.domain.entity.PmContMscTech;
import com.xisc.pm.infra.constant.RuleSetConstants;
import com.xisc.pm.infra.enums.PdRuleItemType;
import io.choerodon.core.exception.CommonException;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * 非轴合同与轴合同的AD控制模式第134位相等匹配检查（A03）
 * 
 * @author jun.zeng
 * 
 * 
 */
@Component
public class CtrlRollMatchRuleExecStrategy implements RuleExecStrategy {

    @Override
    public void afterPropertiesSet() throws Exception {
        RuleExecStrategyFactory.register(PdRuleItemType.CTRL_ROLL_MATCH, this);
    }

    @Override
    public boolean execRule(RuleParamDTO ruleParam) {
        Map<String, Object> bizData = ruleParam.getBizData();
        String orderNo = (String) bizData.get(RuleSetConstants.ORDER_NO);
        String mscLineNo = (String) bizData.get(RuleSetConstants.MSC_LINE_NO);

        PmContMscTech axisPmContMscTech = (PmContMscTech) bizData.get(RuleSetConstants.AXIS_PM_CONT_MSC_TECH);
        PmContMscTech pmContMscTech = (PmContMscTech) bizData.get(RuleSetConstants.PM_CONT_MSC_TECH);

        if (axisPmContMscTech == null || pmContMscTech == null) {
            // 读生产合同池没有找到记录，合同号【order_no】，产线号【msc_line_no】
            throw new CommonException("pm.pm_pd_rule_set_001", orderNo, mscLineNo);
        }
        if(axisPmContMscTech.getCtrlRollCode1() == null && pmContMscTech.getCtrlRollCode1() == null) {
            return true ;
        }
        if(axisPmContMscTech.getCtrlRollCode() == null || pmContMscTech.getCtrlRollCode() == null) {
            // AD控制模式与轴合同不一致
            throw new CommonException("pm.pm_pd_rule_set_053");
        }
        String axisCtrlRollCode = axisPmContMscTech.getCtrlRollCode1() + axisPmContMscTech.getCtrlRollCode34();
        String ctrlRollCode = pmContMscTech.getCtrlRollCode1() + pmContMscTech.getCtrlRollCode34();
        if(!axisCtrlRollCode.equals(ctrlRollCode)) {
            throw new CommonException("pm.pm_pd_rule_set_053");
        }
        return true;
    }

}
