package com.xisc.pm.app.service.strategy.pdrule;

import com.xisc.pm.api.dto.RuleParamDTO;
import com.xisc.pm.app.service.strategy.RuleExecStrategyFactory;
import com.xisc.pm.domain.entity.PmProContProdTtr;
import com.xisc.pm.domain.vo.MmPlateVO;
import com.xisc.pm.infra.constant.RuleSetConstants;
import com.xisc.pm.infra.enums.PdRuleItemType;
import io.choerodon.core.exception.CommonException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Map;

/**
 * 余材钢板长度与合同长度匹配检查（P01）
 */
@Component
public class PlateWidthMatchRuleExecStrategy implements RuleExecStrategy {

    @Override
    public void afterPropertiesSet() throws Exception {
        RuleExecStrategyFactory.register(PdRuleItemType.PLATE_WIDTH_MATCH, this);
    }

    @Override
    public boolean execRule(RuleParamDTO ruleParam) {
        Map<String, Object> bizData = ruleParam.getBizData();
        String orderNo = (String) bizData.get(RuleSetConstants.ORDER_NO);
        String mscLineNo = (String) bizData.get(RuleSetConstants.MSC_LINE_NO);

        MmPlateVO mmPlateVO = (MmPlateVO) bizData.get(RuleSetConstants.MM_PLATE);
        PmProContProdTtr pmProContProdTtr = (PmProContProdTtr) bizData.get(RuleSetConstants.PM_PRO_CONT_PROD_ATTR);

        if (pmProContProdTtr == null || StringUtils.isBlank(pmProContProdTtr.getGiveSampleCode())) {
            // 读生产合同池没有找到记录，合同号【order_no】，产线号【msc_line_no】
            throw new CommonException("pm.pm_pd_rule_set_001", orderNo, mscLineNo);
        }
        if (mmPlateVO == null) {
            throw new CommonException("pm.pm_pd_plate_001");
        }
        BigDecimal orderWidth = pmProContProdTtr.getOrderWidth() == null ? BigDecimal.ZERO
                : pmProContProdTtr.getOrderWidth();
        BigDecimal orderMinWidth = pmProContProdTtr.getWidthMin() == null ? BigDecimal.ZERO
                : pmProContProdTtr.getWidthMin();
        BigDecimal matActWid = mmPlateVO.getMatActWid() == null ? BigDecimal.ZERO : mmPlateVO.getMatActWid();

        if (BigDecimal.ZERO.compareTo(orderMinWidth) != 0 && orderMinWidth.compareTo(matActWid) > 0) {
            // 余材钢板宽度【MAT_ACT_WIDTH】小于合同【order_no】的最小宽度【WIDTH_MIN】。
            throw new CommonException("pm.pm_pd_rule_set_031", orderMinWidth.toPlainString(), orderNo, orderMinWidth.toPlainString());
        }
        if (BigDecimal.ZERO.compareTo(orderMinWidth) == 0 && orderWidth.compareTo(matActWid) > 0) {
            // 余材钢板宽度【MAT_ACT_WIDTH】小于合同【order_no】的订货宽度【ORDER_WIDTH】。
            throw new CommonException("pm.pm_pd_rule_set_032", orderMinWidth.toPlainString(), orderNo, orderMinWidth.toPlainString());
        }
        return true;
    }

}
