package com.xisc.pm.app.service.strategy.pdrule;

import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import com.xisc.pm.api.dto.RuleParamDTO;
import com.xisc.pm.app.service.strategy.RuleExecStrategyFactory;
import com.xisc.pm.domain.entity.PmContMscTech;
import com.xisc.pm.infra.constant.RuleSetConstants;
import com.xisc.pm.infra.enums.PdRuleItemType;

import io.choerodon.core.exception.CommonException;

/**
 * 轧制方向匹配 （C01）
 * @author jun.zeng
 *
 */
@Component
public class RollDirectRuleExecStrategy implements RuleExecStrategy {

    @Override
    public void afterPropertiesSet() throws Exception {
        RuleExecStrategyFactory.register(PdRuleItemType.ROLL_DIRECTION, this);
    }

    @Override
    public boolean execRule(RuleParamDTO ruleParam) {
        Map<String, Object> bizData = ruleParam.getBizData();
        String orderNo = (String) bizData.get(RuleSetConstants.ORDER_NO);
        String mscLineNo = (String) bizData.get(RuleSetConstants.MSC_LINE_NO);

        // 轧制方向
        String rollDirectCode = (String) bizData.get(RuleSetConstants.ROLL_DIRECT_CODE);
        // 产线工艺参数
        PmContMscTech pmContMscTech = (PmContMscTech) bizData.get(RuleSetConstants.PM_CONT_MSC_TECH);
        if (pmContMscTech == null) {
            // 生产合同池没有找到记录，合同号【order_no】，产线号【msc_line_no】。
            throw new CommonException("pm.pm_pd_rule_set_001", orderNo, mscLineNo);
        }
        if (StringUtils.isBlank(pmContMscTech.getRollDirectCode())
                || RuleSetConstants.ZERO.equals(pmContMscTech.getRollDirectCode())
                || pmContMscTech.getRollDirectCode().equals(rollDirectCode)) {
            return true ;
        }
        // 合同【order_no】的轧制方向代码【ROLL_DIRECT_CODE】与指定轧制方向代码【roll_direct_code】不匹配
        throw new CommonException("pm.pm_pd_rule_set_002", orderNo, pmContMscTech.getRollDirectCode() , rollDirectCode);
    }

}
