package com.xisc.pm.app.service.strategy.pdrule;

import com.xisc.pm.api.dto.RuleParamDTO;
import com.xisc.pm.app.service.strategy.RuleExecStrategyFactory;
import com.xisc.pm.domain.entity.PmProContProdTtr;
import com.xisc.pm.domain.vo.MmPlateVO;
import com.xisc.pm.infra.constant.RuleSetConstants;
import com.xisc.pm.infra.enums.PdMatType;
import com.xisc.pm.infra.enums.PdRuleItemType;
import io.choerodon.core.exception.CommonException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * 余材组板牌号比对检查（P01）
 */
@Component
public class SgSignMatchRuleExecStrategy implements RuleExecStrategy {

    @Override
    public void afterPropertiesSet() throws Exception {
        RuleExecStrategyFactory.register(PdRuleItemType.SG_SGIN_MATCH, this);
    }

    @Override
    public boolean execRule(RuleParamDTO ruleParam) {
        Map<String, Object> bizData = ruleParam.getBizData();
        String orderNo = (String) bizData.get(RuleSetConstants.ORDER_NO);
        String mscLineNo = (String) bizData.get(RuleSetConstants.MSC_LINE_NO);
        String matDesignKind = (String) bizData.get(RuleSetConstants.MAT_DESIGN_KIND);

        PmProContProdTtr pmProContProdTtr = (PmProContProdTtr) bizData.get(RuleSetConstants.PM_PRO_CONT_PROD_ATTR);
        MmPlateVO mmPlateVO = (MmPlateVO) bizData.get(RuleSetConstants.MM_PLATE);
        if (pmProContProdTtr == null || StringUtils.isBlank(pmProContProdTtr.getGiveSampleCode())) {
            // 读生产合同池没有找到记录，合同号【order_no】，产线号【msc_line_no】
            throw new CommonException("pm.pm_pd_rule_set_001", orderNo, mscLineNo);
        }
        if (mmPlateVO == null) {
            throw new CommonException("pm.pm_pd_plate_001");
        }
        if (PdMatType.P.getCode().equals(matDesignKind) && !pmProContProdTtr.getSgSign().equals(mmPlateVO.getSgSign())) {
            // 合同【order_no】的牌号【SG_SIGN】与材料的牌号【SG_SIGN】不匹配。
            throw new CommonException("pm.pm_pd_rule_set_033", orderNo, pmProContProdTtr.getSgSign(), mmPlateVO.getSgSign());
        }
        return true;
    }

}
