package com.xisc.pm.app.service.strategy.pdrule;

import java.util.Map;

import org.springframework.stereotype.Component;

import com.xisc.pm.api.dto.RuleParamDTO;
import com.xisc.pm.app.service.strategy.RuleExecStrategyFactory;
import com.xisc.pm.domain.entity.PmProContProdTtr;
import com.xisc.pm.infra.constant.RuleSetConstants;
import com.xisc.pm.infra.enums.PdRuleItemType;

import io.choerodon.core.exception.CommonException;

/**
 * 轴合同和非轴合同二次开坯母材合同匹配校验（A03）
 * 
 * @author jun.zeng
 * 
 *
 * 
 */
@Component
public class TwoRollMatchRuleExecStrategy implements RuleExecStrategy {

    @Override
    public void afterPropertiesSet() throws Exception {
        RuleExecStrategyFactory.register(PdRuleItemType.TWO_ROLL_MATCH, this);
    }

    @Override
    public boolean execRule(RuleParamDTO ruleParam) {
        Map<String, Object> bizData = ruleParam.getBizData();
        String orderNo = (String) bizData.get(RuleSetConstants.ORDER_NO);
        String mscLineNo = (String) bizData.get(RuleSetConstants.MSC_LINE_NO);

        PmProContProdTtr axisPmProContProdTtr = (PmProContProdTtr) bizData
                .get(RuleSetConstants.AXIS_PM_PRO_CONT_PROD_ATTR);
        PmProContProdTtr pmProContProdTtr = (PmProContProdTtr) bizData.get(RuleSetConstants.PM_PRO_CONT_PROD_ATTR);

        if (axisPmProContProdTtr == null || pmProContProdTtr == null) {
            // 读生产合同池没有找到记录，合同号【order_no】，产线号【msc_line_no】
            throw new CommonException("pm.pm_pd_rule_set_001", orderNo, mscLineNo);
        }
        if(axisPmProContProdTtr.getTwoRollFlag() == null && pmProContProdTtr.getTwoRollFlag() == null) {
            return true ;
        }
        if(axisPmProContProdTtr.getTwoRollFlag() == null || pmProContProdTtr.getTwoRollFlag() == null) {
            // 二次开坯母材合同标记与轴合同不一致
            throw new CommonException("pm.pm_pd_rule_set_024");
        }
        if (axisPmProContProdTtr.getTwoRollFlag().longValue() != pmProContProdTtr.getTwoRollFlag().longValue()) {
            // 二次开坯母材合同标记与轴合同不一致
            throw new CommonException("pm.pm_pd_rule_set_024");
        }
        return true;
    }

}
