package com.xisc.pm.domain.entity;

import com.fasterxml.jackson.annotation.JsonInclude;
import io.choerodon.mybatis.annotation.ModifyAudit;
import io.choerodon.mybatis.annotation.VersionAudit;
import io.choerodon.mybatis.domain.AuditDomain;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import org.hzero.export.annotation.ExcelColumn;

import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Objects;


/**
 * 余材组板内部钢种以优充次对照表(PmSlabReplBt)实体类
 *
 * @author 纪文桃 wentao.ji01@hand-china.com
 * @since 2024-07-24 11:30:28
 */

@Getter
@Setter
@ApiModel("余材组板内部钢种以优充次对照表")
@VersionAudit
@ModifyAudit
@JsonInclude(value = JsonInclude.Include.NON_NULL)
@Table(name = "PM_SLAB_REPL_BT")
public class PmSlabReplBt extends AuditDomain implements Serializable {
    private static final long serialVersionUID = -17131747700208192L;

    public static final String FIELD_PSRB_ID = "psrbId";
    public static final String FIELD_BETTER_TO_WORSE_SEQ = "betterToWorseSeq";
    public static final String FIELD_CUR_ST_NO = "curStNo";
    public static final String FIELD_REPL_ST_NO = "replStNo";
    public static final String FIELD_VALID_FLAG = "validFlag";
    public static final String FIELD_REMARK = "remark";
    public static final String FIELD_TENANT_ID = "tenantId";
    public static final String FIELD_ATTRIBUTE1 = "attribute1";
    public static final String FIELD_ATTRIBUTE2 = "attribute2";
    public static final String FIELD_ATTRIBUTE3 = "attribute3";
    public static final String FIELD_ATTRIBUTE4 = "attribute4";
    public static final String FIELD_ATTRIBUTE5 = "attribute5";

    @ApiModelProperty("主键ID")
    @Id
    @GeneratedValue
    private Long psrbId;

    @ApiModelProperty(value = "以优充次顺序码", required = true)
    @NotNull
    private Long betterToWorseSeq;

    @ApiModelProperty(value = "当前出钢记号", required = true)
    @NotBlank
    private String curStNo;

    @ApiModelProperty(value = "替代出钢记号", required = true)
    @NotBlank
    private String replStNo;

    @ApiModelProperty(value = "生效标记", required = true)
    @NotNull
    private String validFlag;

    @Transient
    @ApiModelProperty(value = "生效标记含义")
    private String validFlagDesc;

    @ExcelColumn(zh = "备注", en = "remark", order = 5)
    @ApiModelProperty(value = "备注")
    private String remark;

    @ApiModelProperty(value = "租户ID", required = true)
    @NotNull
    private Long tenantId;

    private String attribute1;

    private String attribute2;

    private String attribute3;

    private String attribute4;

    private String attribute5;

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        PmSlabReplBt that = (PmSlabReplBt) o;
        return Objects.equals(betterToWorseSeq, that.betterToWorseSeq) && Objects.equals(curStNo, that.curStNo) &&
                Objects.equals(replStNo, that.replStNo) && Objects.equals(validFlag, that.validFlag) &&
                Objects.equals(validFlagDesc, that.validFlagDesc) && Objects.equals(remark, that.remark) &&
                Objects.equals(tenantId, that.tenantId);
    }

    @Override
    public int hashCode() {
        return Objects.hash(betterToWorseSeq, curStNo, replStNo, validFlag, validFlagDesc, remark, tenantId);
    }
}
